'-----------------------------------------------------------------------
' <copyright file="FileTreeView.vb" company="Kanazawa-soft-design,LLC.">
'     Copylight(C) 2018 Kanazawa-soft-design,LLC.
' </copyright>
' <author>金澤 宣明</author>
' <summary></summary>
'-----------------------------------------------------------------------

Partial Class MainForm

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ディレクトリツリーの初期処理.
	''' </summary>
	Private Sub FileView_Init()

		' コンピュータ名をディレクトリツリーのルートに登録する.
		Dim root As TreeNode = AddRootNode()

		' ディレクトリツリーにドライブ名を登録する.
		AddDriveName( root )

		' ルートノードを展開して表示する.
		root.Expand()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' コンピュータ名をツリーのルートノードにセットする.
	''' </summary>
	''' <returns>
	''' 生成したルートノード.
	''' </returns>
	Private Function AddRootNode() As TreeNode

		' コンピュータ名をディレクトリツリーのルートに登録する.
		Dim root As new TreeNode( Environment.MachineName )

		' コンピュータ名をツリーのルートノードにセットする.
		FileTree.Nodes.Add( root )

		Return root

	End Function

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ディレクトリツリーにドライブ名を登録する.
	''' </summary>
	''' <param name="root">
	''' ディレクトリツリーのルートノード.
	''' </param>
	Private Sub AddDriveName( root As TreeNode )

		' 論理ドライブ名をすべて取得する.
		Dim stDrives() As String = System.IO.Directory.GetLogicalDrives()

		' 取得した論理ドライブ名をすべて表示する.
		For Each drive As String In stDrives

			' ドライブ名をの最後にあるセパレータ("\")を取り除く.
			Dim drvname = drive.Substring( 0, drive.Length - 1 )

			' ドライブ名をディレクトリツリーに登録する.
			Dim drvnode As New TreeNode( drvname )
			root.Nodes.Add( drvnode )

			' ドライブ名直下のディレクトリが存在するかを調べる.
			AddDirectory( drvnode, drive, true )
		Next

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ディレクトリツリーにディレクトリ名を登録する.
	''' </summary>
	''' <param name="parent">
	''' ディレクトリ名を登録するノード.
	''' </param>
	''' <param name="path">
	''' 該当ノードに対応した絶対パスのディレクトリ名.
	''' </param>
	''' <param name="isDummy">
	''' true = サブディレクトリが存在するかどうかだけ調べる.
	''' </param>
	Private Sub AddDirectory( parent As TreeNode, path As String, isDummy As Boolean )

		path += "\"

		Try
			' サブディレクトリを列挙する.
			For Each dir As String In System.IO.Directory.GetDirectories( path )

				' ディレクトリの属性を取得する.
				Dim di As New System.IO.DirectoryInfo( dir )

				' 隠しディレクトリはツリーに含まない.
				If (di.Attributes And System.IO.FileAttributes.Hidden) <> System.IO.FileAttributes.Hidden Then

					' ディレクトリをツリーに追加する.
					Dim subDir As New TreeNode( di.Name )
					parent.Nodes.Add( subDir )

					' サブディレクトリの存在チェックの場合は、1件検出した時点で処理を終了する.
					If isDummy Then
						Return
					End If

					' サブディレクトリの存在チェックを行う.
					Dim subPath As String = path + subDir.Text
					AddDirectory( subDir, subPath, true )
				End If
			Next

		Catch ex As System.Exception

		End Try

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ディレクトリツリーのディレクトリが展開されるときのイベント処理.
	''' </summary>
	''' <param name="sender">
	''' イベント対象のオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベントパラメータ.
	''' </param>
	Private Sub FileTree_BeforeExpand( sender As Object, e As TreeViewCancelEventArgs ) Handles FileTree.BeforeExpand

		Dim node As TreeNode = e.Node
		If node Is Nothing Then
			Return
		End If

		' 展開されるディレクトリのサブディレクトリを一旦クリアする.
		node.Nodes.Clear()

		' 展開されるノードに対応したディレクトリの絶対パスを取得する.
		Dim path As String = GetAbsolutePath( node )

		If path.Length > 0 Then
			' 展開されるディレクトリのサブディレクトリを表示する.
			AddDirectory( node, path, False )
		Else
			' フルパス名が無い場合はルートの展開なのでドライブ名を表示する.
			AddDriveName( node )
		End If

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ディレクトリツリーのディレクトリの選択が変更されるときのイベント処理.
	''' </summary>
	''' <param name="sender">
	''' イベント対象のオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベントパラメータ.
	''' </param>
	Private Sub FileTree_BeforeSelect( sender As Object, e As TreeViewCancelEventArgs ) Handles FileTree.BeforeSelect

		Dim node As TreeNode = e.Node
		If node Is Nothing Then
			Return
		End If

		' ファイル一覧を更新する.
		UpdateFileList( GetAbsolutePath( node ) )

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' 指定したディレクトリを展開表示する.
	''' </summary>
	''' <param name="path">
	''' 展開するディレクトリの絶対パス.
	''' </param>
	Private Sub ExpandDirectory( path As String )

		' ディレクトリツリーの該当ディレクトリを展開する.
		ExpandAndSelectDirectory( path )

		' ファイル一覧を更新する.
		UpdateFileList( path )
	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' 指定されたノードに対応した絶対パスを取得する.
	''' </summary>
	''' <param name="node">
	''' 絶対パスを取得するノード.
	''' </param>
	''' <returns>
	''' 指定されたノードに対応した絶対パス.
	''' </returns>
	Private Function GetAbsolutePath( node As TreeNode ) As String

		Dim path As String = String.Empty

		Dim parent As TreeNode = node.Parent
		If Not parent Is Nothing Then
			' 親ディレクトリの絶対パスを取得する.
			path += GetAbsolutePath( parent )

			' 親ディレクトリがある場合は連結する為にセパレータ("\")を付ける.
			If path.Length > 0 Then
				path += "\"
			End If
			' 親ディレクトリとファイル名を連結して絶対パス名にする.
			path += node.Text
		Else

		End If

		Return path

	End Function

	'' -----------------------------------------------------------------------
	''' <summary>
	''' 指定されたノードに対応した絶対パスを取得する.
	''' </summary>
	''' <param name="path">
	''' 選択するディレクトリの絶対パス名.
	''' </param>
	Private Sub ExpandAndSelectDirectory( path As String )

		Dim node As TreeNode = FileTree.Nodes(0)

		' IndexOf()検索で最後まで検索するようにパスの最後に検索文字を付加する.
		Dim wkPath As String = path + "\"

		Dim nextPos As Integer
		Dim nowPos	As Integer = 0
		While Not node Is Nothing

			' ディレクトリ名を取り出す.
			nextPos = wkPath.IndexOf( "\", nowPos )
			If nextPos < 0 Then
				Exit While
			End If

			' 子ノードが無い場合は処理を終了する.
			If node.Nodes.Count = 0 Then
				node = Nothing
				Exit While
			End If

			' "\"が連続している場合は読み飛ばす.
			If nextPos = nowPos Then
				nowPos = nextPos + 1
				Continue While
			End If

			' ディレクトリ名を絶対パスから取り出す.
			Dim dirName As String = wkPath.Substring( nowPos, (nextPos - nowPos) )
			nowPos = nextPos + 1

			For Each wkNode As TreeNode In node.Nodes
				' 同じディレクトリ名のノードを見つけたらループを抜ける.
				If dirName.Equals( wkNode.Text, StringComparison.OrdinalIgnoreCase ) Then
					If Not wkNode.IsExpanded Then
						wkNode.Expand()
					End If

					node = wkNode
					Exit For
				End If
			Next
		End While

		' ノードを選択状態にする.
		If Not node Is Nothing Then
			FileTree.SelectedNode = node
			FileTree.Focus()
		End If

	End Sub

End Class
